﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace LINQtoAll
{
  internal class Workers
  {
    private Random random;
    private List<Coworker> wrokers;
    private const string FILE_NAME = @"data\Workers.xml";

    internal List<Coworker> Members
    {
      get { return wrokers; }
    }

    #region initialization

    public Workers()
    {
      random = new Random();
      wrokers = new List<Coworker>();
      GenerateMembers(20);
    }

    /// <summary>
    /// Fills Members list with required items count.
    /// </summary>
    /// <param name="count">How many Coworkers to add in list.</param>
    public void GenerateMembers(int count)
    {
      for (int i = 0; i <= count; i++)
      {
        string name = GetRandomName(5);
        string surName = GetRandomName(10);
        wrokers.Add(new Coworker { Name = name, SurName = surName });
      }
    }

    /// <summary>
    /// Creates string from random nummbers.
    /// </summary>
    /// <param name="length">return string length as characters count</param>
    /// <returns>Upper case random string.</returns>
    private string GetRandomName(int length)
    {
      string retstr = "";

      for (int i = 0; i <= length; i++)
      {
        int ch = random.Next(0, 25) + 65;
        retstr += ((char)ch).ToString();
      }

      return retstr;
    }

    #endregion

    #region LINQ methods

    public List<Coworker> GetNames()
    {  //klasicky select
      var sortedWorkers =
           from Worker in wrokers
           orderby Worker.FullName
           select Worker;

      return sortedWorkers.ToList();
    }


    public List<Coworker> GetTop5bySurName()
    {
      //Take vraci neserazeny list
      return GetNames().Take(5).ToList();
    }

    public List<Coworker> GroupbySurName()
    {
      var orderGroups =
           from p in wrokers
           group p by p.Name.Substring(0, 1) into g
           select new { grupa = g };

      //Take vraci list pro kazdou skupinu
      return orderGroups.ToList()[0].grupa.ToList();

    }

    public bool GetAny()
    {
      return wrokers.Any(w => w.SurName.Contains("ei"));
    }

    #region XML

    public List<string> LoadXML()
    {
      XDocument xmlWorkers = XDocument.Load(FILE_NAME);

      var q = from c in xmlWorkers.Descendants("contact")
              where (int)c.Attribute("contactId") < 4
              select (string)c.Element("firstName") + " " +
                 (string)c.Element("lastName");

      return q.ToList();
    }

    public void AddIntoXML()
    {
      XDocument xmlWorkers = XDocument.Load(FILE_NAME);
      int newIndex = xmlWorkers.Descendants("contact").ToList().Count + 1;
      xmlWorkers.Root.Add(
              new XElement("contact",
                  new XAttribute("contactId", newIndex),
                  new XElement("firstName", "AAA"),
                  new XElement("lastName", "BBB")
              ));
      xmlWorkers.Save(FILE_NAME);

    }
    #endregion

    //ostatni prikazy na dotazy viz. linqsamples v instalacni slozce
    #endregion

  }
}
